function out = loglikeb(xx,T)

% Calculates the log likelihood 

% The model in state-space form is given by
%
% y(t) = C(t) + P(t) y(t-1) + D(t) e(t)      State Eq
%
% z(t) = H y(t) + v(t)                        Measurement Eq 
%
% Dimensions and Definitions : 
%  
%                y(t)    is n x 1     state 
%                Zobs(t) is nobs x 1  observable
%                e(t)    is l x 1     structural shocks
%                v(t)    is nobs x 1     measurement errors
%
%                V = E[v(t)v(t)'] is nobs x nobs
%
%                Omega = E[e(t)e(t)'] is nshocks x nshocks
%
%                C(t) is (n-1) x 1
%
%                P(t) is (n-1) x (n-1)
%
%                D(t) is (n-1) x nshocks
%
%                H is nobs x (n-1)
% 
%
% Last Modified: 18-Nov-2014
%                Mariano Kulish

global Zobs % Declares the data global

penalty = 1e12;

%% Date breaks and sample size
T1  = 91; % Set to first quarter of Volcker regime 1979Q4
Tb  = T(1); % Estimate
T_sig = T(2); % Date break for variances.

%%
samplesize = length(Zobs);
[nobs, ~] = size(Zobs);
V = zeros(nobs); % Var-cov of measurement error

%% Variables and shocks
% Order of in y(t)

ij = 1;

r          = ij; ij = ij + 1;
yhat       = ij; ij = ij + 1;
g          = ij; ij = ij + 1;
a          = ij; ij = ij + 1; 
e          = ij; ij = ij + 1;
ppi_target = ij; ij = ij + 1;
x          = ij; ij = ij + 1; 
ppi        = ij; ij = ij + 1; 
ppi1       = ij; ij = ij + 1;
ppi2       = ij; ij = ij + 1;
ppi3       = ij; ij = ij + 1;
ppi4       = ij; ij = ij + 1;
Eppi_obs   = ij; ij = ij + 1;
eta_obs    = ij; ij = ij + 1; 

const = ij; 

n = ij; % Dimension of y(t)

% Order of Shocks in e(t)
%   1     2      3     4      5
%  er(t) ea(t), ee(t), ez(t) e_eta(t);
ij = 1;
er = ij; ij = ij + 1;
ea = ij; ij = ij + 1; 
ee = ij; ij = ij + 1; 
ez = ij; ij = ij + 1; 
e_target = ij; ij = ij + 1; 
e_eta_obs = ij;  
% Dimension of et(t)
l = ij;

%% Parameters values
ij = 1;

% Standard Errors
sig_r      = xx(ij)/100; ij = ij + 1; % 0.0017;
sig_a      = xx(ij)/100; ij = ij + 1; %0.0100;
sig_e      = xx(ij)/100; ij = ij + 1; %0.0018;
sig_z      = xx(ij)/100; ij = ij + 1; %0.0040;
sig_target = xx(ij)/100; ij = ij + 1; %0.0040;
sig_eta    = xx(ij)/100; ij = ij + 1; %0.001;

mu         = xx(ij)      ; ij = ij + 1;
% Monetary policy

rho_r = xx(ij); ij = ij + 1; %0.7;
rho_pi = xx(ij); ij = ij + 1; %0.3;
rho_g = xx(ij); ij = ij + 1; %0.1;
rho_x = 0;      % xx(ij); ij = ij + 1; %0.05;

rho_pi_f = xx(ij); ij = ij + 1; %0.5;
rho_g_f  = xx(ij); ij = ij + 1; %0.1;

% Households and firms 
beta  = 0.9975;
alpha = 0; % 
psi   = 0.1;  % 
omega = 0.05; %

% Exogenous processes
rho_a      = xx(ij); ij = ij + 1; %0.85;
rho_e      = xx(ij); ij = ij + 1; %0.75;
rho_eta    = xx(ij); ij = ij + 1; %0.5;
rho_target = xx(ij); ij = ij + 1; %0.5;

% Steady state constants
g_ss    = xx(ij)/100; ij = ij + 1; %0.006; % Steady state quarterly growth rate of output per capita
ppi_ss  = xx(ij)/100; ij = ij + 1; %0.005; % Steady state quarterly rate of price inflation
r_ss    = ppi_ss + g_ss - log(beta); % Steady state nominal interest rate.

g_ss_f   = g_ss; % xx(ij)/100; ij = ij + 1; %0.003;
ppi_ss_f = xx(ij)/100;            %0.010;
r_ss_f   = ppi_ss_f + g_ss_f - log(beta);

mu_target  = 1;
mu_target_f = 0;
rho_target_f = rho_target;

Omega = diag([mu*sig_r, mu*sig_a, mu*sig_e, mu*sig_z, mu*sig_eta, mu*sig_target].^2);

Omegaf = diag([sig_r, sig_a, sig_e, sig_z, sig_eta, sig_target].^2);



%% Regime 1

A0 = zeros(n,n);
A1 = zeros(n,n);
B0 = zeros(n,n);
D0 = zeros(n,l);

eqn = 1;

% Euler equation
% x(t) = (r_ss-ppi_ss) - (r(t) - Et pi(t+1)) + Et x(t+1) +
% (1-omega)(1-rho_a)a(t)

A0(eqn,r)     = 1;
A0(eqn,a)     = -(1-rho_a)*(1-omega);
A0(eqn,x)     = 1;
A0(eqn,const) = ppi_ss - r_ss;
B0(eqn,x)     = 1;
B0(eqn,ppi)   = 1;

eqn = eqn + 1;

% Phillips Curve
% (1+beta*alpha) pi(t) = (1+beta*alpha-alpha-beta)*ppi_ss + alpha pi(t-1) + beta*Et pi(t+1) + psi x(t) - ehat(t))

A0(eqn,e) = 1;
A0(eqn,x) = -psi;
A0(eqn,ppi) = 1 + beta*alpha;
A0(eqn,const) = -(1-beta+beta*alpha-alpha)*ppi_ss;
A1(eqn,ppi) = alpha;
B0(eqn,ppi) = beta;

eqn = eqn + 1;

% Monetary Policy Rule
% r(t) = (1-rho_r)*rss -(rho_pi)*ppi_ss - rho_g*g_ss + rho_r*r(t-1) +
% rho_pi*pi(t) + rho_E Et pi(t+1)
% + rho_g*g(t) + rho_x*x(t) + er(t)

A0(eqn,r) = 1;
A0(eqn,g) = -rho_g;
A0(eqn,x) = -rho_x;
A0(eqn,ppi) = -rho_pi;
A0(eqn,ppi_target) = rho_pi;
A0(eqn,const) = -((1-rho_r)*r_ss-rho_g*g_ss);
A1(eqn,r) = rho_r;
D0(eqn,er) = 1;

eqn = eqn + 1;

% Output gap
% x(t) = yhat(t) - omega a(t)

A0(eqn,yhat) = -1;
A0(eqn,a) = omega;
A0(eqn,x) = 1;

eqn = eqn + 1;

% Growth
% g(t) = g_ss + yhat(t) - yhat(t-1) + e_z(t)

A0(eqn,yhat) = -1;
A0(eqn,g) = 1;
A0(eqn,const) = -g_ss;
A1(eqn,yhat) = -1;
D0(eqn,ez) = 1;

eqn = eqn + 1;


% ppi_target
% pitarget(t) = (1-rho_target) ppi_ss + rho_target* pitarget(t-1) + e_target(t)

A0(eqn,ppi_target) = 1;
A0(eqn,const) = -(1-rho_target)*ppi_ss;
A1(eqn,ppi_target) = rho_target;
D0(eqn,e_target) = mu_target;

eqn = eqn + 1;

% ppi1(t) = Et ppi(t+1)

A0(eqn,ppi1) = 1;
B0(eqn,ppi)  = 1;

eqn = eqn + 1;

% ppi2(t) = Et ppi1(t+1)

A0(eqn,ppi2) = 1;
B0(eqn,ppi1)  = 1;

eqn = eqn + 1;

% ppi3(t) = Et ppi2(t+1)

A0(eqn,ppi3) = 1;
B0(eqn,ppi2)  = 1;

eqn = eqn + 1;

% ppi4(t) = Et ppi3(t+1)

A0(eqn,ppi4) = 1;
B0(eqn,ppi3)  = 1;

eqn = eqn + 1;

% Eppi_obs(t) = 1/4 * {ppi1(t) + ppi2(t) + ppi3(t) + ppi4(t)} + eta_obs(t)

A0(eqn,Eppi_obs) = 1;
A0(eqn,ppi1)     = -1/4;
A0(eqn,ppi2)     = -1/4;
A0(eqn,ppi3)     = -1/4;
A0(eqn,ppi4)     = -1/4;
A0(eqn,eta_obs)  = -1;

eqn = eqn + 1;

% Preference shocks
% a(t) = rho_a*a(t-1) + e_a(t)
A0(eqn,a) = 1;
A1(eqn,a) = rho_a;
D0(eqn,ea) = 1;

eqn = eqn + 1;

% Mark-up shock
% e(t) = rho_e*e(t-1) + e_e(t)

A0(eqn,e) = 1;
A1(eqn,e) = rho_e;
D0(eqn,ee) = 1;

eqn = eqn + 1;

% eta_obs
% eta_obs(t) = rho_e*eta_obs(t-1) + e_eta_obs(t)

A0(eqn,eta_obs) = 1;
A1(eqn,eta_obs) = rho_eta;
D0(eqn,e_eta_obs) = 1;

eqn = eqn + 1;

% Constant
% const(t) = const(t-1)

A0(eqn,const) = 1;
A1(eqn,const) = 1;

% Because ar(1)s are in y(t)

Gamma = zeros(l);
D2 = zeros(n,l);

% Compute Solution
[Q, G, unique, rc, iter] = smatsbp(A0,A1,B0,D0,D2,Gamma);
if unique ~= 1
   out = penalty;
   return
end
% Steady state
P = Q(1:end-1,1:end-1);
C = Q(1:end-1,end);
y_ss = (eye(n-1)-P)\C;
y_ss(n) = 1;


%% Regime 2
A0f = zeros(n,n);
A1f = zeros(n,n);
B0f = zeros(n,n);
D0f = zeros(n,l);

eqn = 1;

% Euler equation
% x(t) = (r_ss-ppi_ss) - (r(t) - Et pi(t+1)) + Et x(t+1) +
% (1-omega)(1-rho_a)a(t)

A0f(eqn,r)     = 1;
A0f(eqn,a)     = -(1-rho_a)*(1-omega);
A0f(eqn,x)     = 1;
A0f(eqn,const) = ppi_ss_f - r_ss_f;
B0f(eqn,x)     = 1;
B0f(eqn,ppi)   = 1;

eqn = eqn + 1;

% Phillips Curve
% (1+beta*alpha) pi(t) = (1+beta*alpha-alpha-beta)*ppi_ss + alpha pi(t-1) + beta*Et pi(t+1) + psi x(t) - ehat(t))

A0f(eqn,e) = 1;
A0f(eqn,x) = -psi;
A0f(eqn,ppi) = 1 + beta*alpha;
A0f(eqn,const) = -(1-beta+beta*alpha-alpha)*ppi_ss_f;
A1f(eqn,ppi) = alpha;
B0f(eqn,ppi) = beta;

eqn = eqn + 1;

% Monetary Policy Rule
% r(t) = (1-rho_r)*rss -(rho_pi)*ppi_ss - rho_g*g_ss + rho_r*r(t-1) +
% rho_pi*pi(t) + rho_E Et pi(t+1)
% + rho_g*g(t) + rho_x*x(t) + er(t)

A0f(eqn,r) = 1;
A0f(eqn,g) = -rho_g_f;
A0f(eqn,x) = -rho_x;
A0f(eqn,ppi) = -rho_pi_f;
A0f(eqn,ppi_target) = rho_pi_f;
A0f(eqn,const) = -((1-rho_r)*r_ss_f-rho_g_f*g_ss_f);
A1f(eqn,r) = rho_r;
D0f(eqn,er) = 1;

eqn = eqn + 1;

% Output gap
% x(t) = yhat(t) - omega a(t)

A0f(eqn,yhat) = -1;
A0f(eqn,a) = omega;
A0f(eqn,x) = 1;

eqn = eqn + 1;

% Growth
% g(t) = g_ss + yhat(t) - yhat(t-1) + e_z(t)

A0f(eqn,yhat) = -1;
A0f(eqn,g) = 1;
A0f(eqn,const) = -g_ss_f;
A1f(eqn,yhat) = -1;
D0f(eqn,ez) = 1;

eqn = eqn + 1;


% ppi_target
% pitarget(t) = (1-rho_target) ppi_ss + rho_target* pitarget(t-1) + e_target(t)

A0f(eqn,ppi_target) = 1;
A0f(eqn,const) = -(1-rho_target_f)*ppi_ss_f;
A1f(eqn,ppi_target) = rho_target_f;
D0f(eqn,e_target) = mu_target_f;

eqn = eqn + 1;

% ppi1(t) = Et ppi(t+1)

A0f(eqn,ppi1) = 1;
B0f(eqn,ppi)  = 1;

eqn = eqn + 1;

% ppi2(t) = Et ppi1(t+1)

A0f(eqn,ppi2) = 1;
B0f(eqn,ppi1)  = 1;

eqn = eqn + 1;

% ppi3(t) = Et ppi2(t+1)

A0f(eqn,ppi3) = 1;
B0f(eqn,ppi2)  = 1;

eqn = eqn + 1;

% ppi4(t) = Et ppi3(t+1)

A0f(eqn,ppi4) = 1;
B0f(eqn,ppi3)  = 1;

eqn = eqn + 1;

% Eppi_obs(t) = 1/4 * {ppi1(t) + ppi2(t) + ppi3(t) + ppi4(t)} + eta_obs(t)

A0f(eqn,Eppi_obs) = 1;
A0f(eqn,ppi1)     = -1/4;
A0f(eqn,ppi2)     = -1/4;
A0f(eqn,ppi3)     = -1/4;
A0f(eqn,ppi4)     = -1/4;
A0f(eqn,eta_obs)  = -1;

eqn = eqn + 1;

% Preference shocks
% a(t) = rho_a*a(t-1) + e_a(t)
A0f(eqn,a) = 1;
A1f(eqn,a) = rho_a;
D0f(eqn,ea) = 1;

eqn = eqn + 1;

% Mark-up shock
% e(t) = rho_e*e(t-1) + e_e(t)

A0f(eqn,e) = 1;
A1f(eqn,e) = rho_e;
D0f(eqn,ee) = 1;

eqn = eqn + 1;

% eta_obs
% eta_obs(t) = rho_e*eta_obs(t-1) + e_eta_obs(t)

A0f(eqn,eta_obs) = 1;
A1f(eqn,eta_obs) = rho_eta;
D0f(eqn,e_eta_obs) = 1;

eqn = eqn + 1;

% Constant
% const(t) = const(t-1)

A0f(eqn,const) = 1;
A1f(eqn,const) = 1;

% Because ar(1)s are in y(t)

Gamma = zeros(l);
D2 = zeros(n,l);


% Compute Solution
[Qf, Gf, uniquef, rc, iter] = smatsbp(A0f,A1f,B0f,D0f,D2,Gamma);
if uniquef ~= 1
   out = penalty;
   return
end
% Steady state
Pf = Qf(1:end-1,1:end-1);
Cf = Qf(1:end-1,end);
y_ssf = (eye(n-1)-Pf)\Cf;
y_ssf(n) = 1;

% Additional matrices for anticipated transition
Af = A0f\A1f;
Bf = A0f\B0f;
Df = A0f\D0f;

%% Beliefs different to reality

Qb = (A0f-B0f*Q)\A1f;
Gb = (A0f-B0f*Q)\D0f;

%% Construct sequence of reduced form matrices

Qt = zeros(n,n,samplesize+1); % sequence of reduced form matrices
Gt = zeros(n,l,samplesize+1); % sequence of reduced form matrices

for t = 1:T1-1
    Qt(:,:,t) = Q;
    Gt(:,:,t) = G;
end
        
for t = T1:Tb-1
    Qt(:,:,t) = Qb;
    Gt(:,:,t) = Gb;
end
        
for t = Tb:samplesize
    Qt(:,:,t) = Qf;
    Gt(:,:,t) = Gf;
end

%% Kalman Filter

Ct = zeros(n-1,1,samplesize+1);
Pt = zeros(n-1,n-1,samplesize+1);
Dt = zeros(n-1,l,samplesize+1);
OM = zeros(l,l,samplesize);

for t = 1:T_sig-1
    OM(:,:,t) = Omega;
end
for t = T_sig:samplesize
    OM(:,:,t) = Omegaf;
end

for t = 1:samplesize
    Ct(:,:,t) = Qt(1:n-1,end,t);
    Pt(:,:,t) = Qt(1:n-1,1:n-1,t);
    Dt(:,:,t) = Gt(1:n-1,:,t);
end

% Measurement equation
% Zobs = [Eppi_obs, r_obs , ppicore_obs, g_obs];

H = zeros(nobs,n-1);
H(1,Eppi_obs) = 1;
H(2,r)        = 1;
H(3,ppi)      = 1;
H(4,g)        = 1;

W = zeros(nobs-1,nobs);
W(:,2:end) = eye(nobs-1); % removes the first observable (Eppi_obs).

HW = W*H;

T_mich = 84; % Corresponds to quarter 1978q1 in Zobs (Michigan data in inflation expectations starts)

%%

DOmega = Dt(:,:,1)*Omega*Dt(:,:,1)';

Sigma = (eye((n-1)^2)-(kron(Pt(:,:,1),Pt(:,:,1))))\DOmega(:); % eye(nn-1) because I remove the constant

Sigma = reshape(Sigma,n-1,n-1);

% Construct tensor to store the l x l matrices P(t+1/t)

    St = zeros(n-1,n-1,samplesize);
    St(:,:,1) = Sigma; % Initial condition
 
 for t=1:(T_mich-1)
     St(:,:,t+1) = Pt(:,:,t+1)*( St(:,:,t)- St(:,:,t)* HW' / ( HW*St(:,:,t)*HW' + W*V*W' ) * HW*St(:,:,t))* Pt(:,:,t+1)' + Dt(:,:,t+1)*OM(:,:,t)*Dt(:,:,t+1)';
 end   
    
 for t=T_mich:(samplesize-1)
     St(:,:,t+1) = Pt(:,:,t+1)*( St(:,:,t)- St(:,:,t)* H' /( H*St(:,:,t)*H' + V ) * H*St(:,:,t))* Pt(:,:,t+1)' + Dt(:,:,t+1)*OM(:,:,t)*Dt(:,:,t+1)';
 end

 Kt  = zeros(n-1,nobs,samplesize);
 KtW = zeros(n-1,nobs-1,samplesize); 

 for t=1:(T_mich-1)
     KtW(:,:,t) = (Pt(:,:,t+1) * St(:,:,t) * HW') / (HW * St(:,:,t) * HW' + W * V * W');
 end
 
 for t=T_mich:samplesize
     Kt(:,:,t) = (Pt(:,:,t+1) * St(:,:,t) * H') / (H * St(:,:,t) * H' + V);
 end

% Construct series for the forecasts y(t+1/t) 

 y_hat = zeros(n-1,samplesize+1);
 y_hat(:,1) = ((eye(n-1) - Pt(:,:,1)))\Ct(:,:,1); 

 for t = 1:(T_mich-1)
     y_hat(:,t+1) = Ct(:,:,t+1) + Pt(:,:,t+1)* y_hat(:,t) + KtW(:,:,t)*(W * Zobs(:,t) - HW * y_hat(:,t));
 end
 
 for t=T_mich:samplesize
     y_hat(:,t+1) = Ct(:,:,t+1) + Pt(:,:,t+1)* y_hat(:,t) + Kt(:,:,t)*( Zobs(:,t) - H * y_hat(:,t));
 end

%% Loglikelihood 
U = zeros(nobs,samplesize);
UW = zeros(nobs-1,samplesize);
out = 0;

for t = 1:(T_mich-1)
        UW(:,t) = W*Zobs(:,t) - HW * y_hat(:,t);
        out = out + (nobs/2)*log(2*pi)+(1/2)*log(det(HW*St(:,:,t)*HW'+W*V*W'))+(1/2)*(UW(:,t)'/(HW*St(:,:,t)*HW'+W*V*W')*UW(:,t));
end
    
for t = T_mich:samplesize;
    U(:,t) = Zobs(:,t) - H * y_hat(:,t);
    out = out + (nobs/2)*log(2*pi)+(1/2)*log(det(H*St(:,:,t)*H'+V))+(1/2)*(U(:,t)'/(H*St(:,:,t)*H'+V)*U(:,t));
end

if  abs(imag(out)) > 0
    out = penalty;
end 
